"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var url = require("url");
var jsonschema_1 = require("jsonschema");
var schema = require("./config/schema.json");
var pattern = function (file) { return ({
    pattern: file,
    included: true,
    served: true,
    watched: false
}); };
var framework = function (config) {
    config.files.push(pattern(path.resolve(__dirname, "config/default.json")), pattern(path.resolve(__dirname, "adapter/index.js")));
    config.beforeMiddleware = config.beforeMiddleware || [];
    config.beforeMiddleware.push("viewport");
    config.preprocessors = config.preprocessors || {};
    config.preprocessors[path.resolve(__dirname, "config/default.json")] = ["viewport"];
};
framework.$inject = ["config"];
var middleware = function () {
    return function (req, res, next) {
        var uri = url.parse(req.url, true);
        if (uri.pathname !== "/debug.html" ||
            typeof uri.query.embed !== "undefined")
            return next();
        var debug = path.resolve(__dirname, "static/debug.html");
        fs.readFile(debug, function (err, data) {
            if (err)
                return next(err);
            res.writeHead(200, { "Content-Type": "text/html" });
            res.end(data.toString()
                .replace("%X_UA_COMPATIBLE%", '<meta http-equiv="X-UA-Compatible" content="' +
                uri.query["x-ua-compatible"] +
                '" />'), "utf-8");
        });
    };
};
var preprocessor = function (viewport, client) {
    if (viewport && typeof viewport !== "object")
        throw new TypeError("Invalid viewport configuration: " + viewport);
    return function (content, file, done) {
        var config = Object.assign(JSON.parse(content), viewport);
        var result = jsonschema_1.validate(config, schema);
        if (result.errors.length)
            throw new TypeError("Invalid viewport configuration: " + result.errors[0].stack);
        if (config.context === "#context" && !client.useIframe)
            throw new Error("Invalid configuration: client.useIframe " +
                "must be set to true or a different context selector must be given");
        file.path += ".js";
        done("window.__viewport__ = " + JSON.stringify(config, undefined, 2));
    };
};
preprocessor.$inject = ["config.viewport", "config.client"];
module.exports = {
    "framework:viewport": ["factory", framework],
    "middleware:viewport": ["factory", middleware],
    "preprocessor:viewport": ["factory", preprocessor]
};
//# sourceMappingURL=index.js.map